package net.javase.wechat.api;

import net.javase.wechat.enums.WxApiEnum;
import net.javase.wechat.kit.WxApiKit;
import net.javase.wechat.model.req.GetPaidUnionIdReq;
import net.javase.wechat.model.resp.auth.*;

import java.util.HashMap;
import java.util.Map;

/**
 * 微信公众平台API
 *
 * @author Frank
 */
public class WxApi {

	/**
	 * 获取AccessToken
	 *
	 * @param appId  APPID
	 * @param secret Secret
	 * @return {@link AccessTokenResp}
	 */
	public static AccessTokenResp getAccessToken(String appId, String secret) {
		Map<String, Object> req = new HashMap<>();
		req.put("grant_type", "client_credential");
		req.put("appid", appId);
		req.put("secret", secret);
		return WxApiKit.getRequest(WxApiKit.getUrl(WxApiEnum.ACCESS_TOKEN), req, AccessTokenResp.class);
	}


	/**
	 * 获取AccessToken
	 *
	 * @param appId  AppId
	 * @param secret Secret
	 * @param code   code
	 * @return {@link AccessTokenResp}
	 */
	public static AccessTokenResp getOauthToken(String appId, String secret, String code) {
		return getOauthToken(appId, secret, code, "authorization_code");
	}

	/**
	 * getOauth2AccessToken
	 *
	 * @param appId     AppId
	 * @param secret    Secret
	 * @param code      code
	 * @param grantType grantType
	 * @return {@link AccessTokenResp}
	 */
	public static AccessTokenResp getOauthToken(String appId, String secret, String code, String grantType) {
		Map<String, Object> query = new HashMap<>();
		query.put("appid", appId);
		query.put("secret", secret);
		query.put("code", code);
		query.put("grant_type", grantType);
		return WxApiKit.getRequest(WxApiKit.getUrl(WxApiEnum.OAUTH_ACCESS_TOKEN), query, AccessTokenResp.class);
	}


	/**
	 * 获取用户个人信息
	 *
	 * @param accessToken oauth2 accessToken
	 * @param openId      openId
	 * @param lang        国家地区语言版本 zh_CN 简体，zh_TW 繁体，en 英语，默认为 zh-CN
	 * @return {@link UserInfoResp}
	 */
	public static UserInfoResp getUserInfo(String accessToken, String openId, String lang) {
		Map<String, Object> query = new HashMap<>();
		query.put("access_token", accessToken);
		query.put("openid", openId);
		query.put("lang", lang);
		return WxApiKit.getRequest(WxApiKit.getUrl(WxApiEnum.USER_INFO), query, UserInfoResp.class);
	}

	/**
	 * 小程序登录
	 *
	 * @param appId  AppId
	 * @param secret Secret
	 * @param code   通过 wx.login 接口获得临时登录凭证 code
	 * @return {@link Code2SessionResp}
	 */
	public static Code2SessionResp code2Session(String appId, String secret, String code) {
		Map<String, Object> query = new HashMap<>();
		query.put("appid", appId);
		query.put("secret", secret);
		query.put("js_code", code);
		query.put("grant_type", "authorization_code");
		return WxApiKit.getRequest(WxApiKit.getUrl(WxApiEnum.CODE_SESSION), query, Code2SessionResp.class);
	}

	/**
	 * 获取用户个人信息
	 *
	 * @param accessToken oauth2 accessToken
	 * @param openId      openId
	 * @return {@link UserInfoResp}
	 */
	public static UserInfoResp getUserInfo(String accessToken, String openId) {
		return getUserInfo(accessToken, openId, "zh-CN");
	}

	/**
	 * 获取用户个人信息
	 *
	 * @param accessTokenResp {@link AccessTokenResp} AccesToken响应对象
	 * @return {@link UserInfoResp}
	 */
	public static UserInfoResp getUserInfo(AccessTokenResp accessTokenResp) {
		return getUserInfo(accessTokenResp.getAccessToken(), accessTokenResp.getOpenid());
	}


	/**
	 * 用户支付完成后，获取该用户的 UnionId <br>
	 * 注意：调用前需要用户完成支付，且在支付后的五分钟内有效。
	 *
	 * @param req 请求参数
	 * @return {@link PaidUnionIdResp} 响应信息
	 */
	public static PaidUnionIdResp getPaidUnionId(GetPaidUnionIdReq req) {
		return WxApiKit.getRequest(WxApiKit.getUrl(WxApiEnum.GET_PAID_UNIONID), req.toMap(), PaidUnionIdResp.class);
	}


	/**
	 * 获取JsApiTicket
	 *
	 * @param accessToken AccessToken
	 * @return {@link JsApiTicketResp}
	 */
	public static JsApiTicketResp getJsApiTicket(String accessToken) {
		Map<String, Object> query = new HashMap<>();
		query.put("access_token", accessToken);
		query.put("type", "jsapi");
		return WxApiKit.getRequest(WxApiKit.getUrl(WxApiEnum.JSAPI_TICKET), query, JsApiTicketResp.class);
	}


}
