package net.javase.wechat.api;

import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import net.javase.wechat.bean.live.Goods;
import net.javase.wechat.bean.live.LiveRoom;
import net.javase.wechat.enums.WxApiEnum;
import net.javase.wechat.kit.WxApiKit;
import net.javase.wechat.model.req.live.*;
import net.javase.wechat.model.resp.BaseResp;
import net.javase.wechat.model.resp.goods.AuditResp;
import net.javase.wechat.model.resp.goods.GoodsAddBaseResp;
import net.javase.wechat.model.resp.goods.GoodsInfoResp;
import net.javase.wechat.model.resp.live.*;

import java.util.HashMap;
import java.util.Map;

/**
 * 【小程序直播】API
 *
 * @author Frank
 */
@Slf4j
public class XcxLiveApi {

	/**
	 * 创建直播间，创建成功后将在直播间列表展示
	 *
	 * @param accessToken AccessToken
	 * @param room        直播间信息
	 * @return {@link RoomCreateResp}
	 */
	public static RoomCreateResp create(String accessToken, LiveRoom room) {
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_CREATE, accessToken), room.toJson(), RoomCreateResp.class);
	}


	/**
	 * 编辑直播间
	 *
	 * @param accessToken AccessToken
	 * @param room        直播间信息
	 * @return {@link BaseResp}
	 */
	public static BaseResp editRoom(String accessToken, LiveRoom room) {
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_EDIT_ROOM, accessToken), room.toJson(), BaseResp.class);
	}


	/**
	 * 删除直播间
	 *
	 * @param accessToken AccessToken
	 * @param roomId      直播间ID
	 * @return {@link BaseResp}
	 */
	public static BaseResp delLiveRoom(String accessToken, int roomId) {
		JSONObject req = new JSONObject();
		req.put("id", roomId);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_DEL_ROOM, accessToken), req.toJSONString(), BaseResp.class);
	}


	/**
	 * 获取直播间列表及直播间信息
	 *
	 * @param accessToken AccessToken
	 * @param start       起始房间，0表示从第1个房间开始拉取
	 * @param limit       每次拉取的房间数量，建议100以内
	 * @return {@link LiveInfoResp}
	 */
	public static LiveInfoResp getLiveInfo(String accessToken, int start, int limit) {
		JSONObject req = new JSONObject();
		req.put("start", start);
		req.put("limit", limit);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GET_INFO, accessToken), req.toJSONString(), LiveInfoResp.class);
	}


	/**
	 * 获取已结束直播间的回放源视频（一般在直播结束后10分钟内生成，源视频无评论等内容）
	 *
	 * @param accessToken AccessToken
	 * @param roomId      直播间ID
	 * @param start       起始拉取视频，0表示从第一个视频片段开始拉取
	 * @param limit       每次拉取的数量，建议100以内
	 * @return {@link ReplayResp}
	 */
	public static ReplayResp getReplay(String accessToken, int roomId, int start, int limit) {
		JSONObject req = new JSONObject();
		req.put("action", "get_replay");
		req.put("room_id", roomId);
		req.put("start", start);
		req.put("limit", limit);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GET_INFO, accessToken), req.toJSONString(), ReplayResp.class);
	}

	/**
	 * 往指定直播间导入已入库的商品
	 *
	 * @param accessToken AccessToken
	 * @param roomId      直播间ID
	 * @param ids         商品ID
	 * @return {@link BaseResp}
	 */
	public static BaseResp addLiveGoods(String accessToken, int roomId, int... ids) {
		JSONObject req = new JSONObject();
		req.put("roomId", roomId);
		req.put("ids", ids);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_ADD_GOODS_TO_ROOM, accessToken), req.toJSONString(), BaseResp.class);
	}


	/**
	 * 获取直播间推流地址
	 *
	 * @param accessToken AccessToken
	 * @param roomId      直播间ID
	 * @return {@link LivePushResp}
	 */
	public static LivePushResp getLivePush(String accessToken, int roomId) {
		Map<String, Object> req = new HashMap<>();
		req.put("access_token", accessToken);
		req.put("roomId", roomId);
		return WxApiKit.getRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GET_PUSH_URL), req, LivePushResp.class);
	}


	/**
	 * 获取直播间分享二维码
	 *
	 * @param accessToken AccessToken
	 * @param roomId      直播间ID
	 * @param params      自定义参数 记得URLEncode
	 * @return {@link LiveShareCodeResp}
	 */
	public static LiveShareCodeResp getShareCode(String accessToken, Integer roomId, String params) {
		Map<String, Object> req = new HashMap<>();
		req.put("access_token", accessToken);
		req.put("roomId", roomId);
		req.put("params", params);
		return WxApiKit.getRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GET_SHARE_CODE), req, LiveShareCodeResp.class);
	}

	/**
	 * 添加管理直播间小助手
	 *
	 * @param accessToken accessToken
	 * @param req         请求信息
	 * @return {@link BaseResp}
	 */
	public static BaseResp addAssistant(String accessToken, AddAssistantReq req) {
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_ADD_ASSISTANT, accessToken), req.toJson(), BaseResp.class);
	}

	/**
	 * 修改管理直播间小助手
	 *
	 * @param accessToken accessToken
	 * @param req         请求信息
	 * @return {@link BaseResp}
	 */
	public static BaseResp modifyAssistant(String accessToken, EditAssistantReq req) {
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_MODIFY_ASSISTANT, accessToken), req.toJson(), BaseResp.class);
	}


	/**
	 * 删除管理直播间小助手
	 *
	 * @param accessToken accessToken
	 * @param req         请求信息
	 * @return {@link BaseResp}
	 */
	public static BaseResp removeAssistant(String accessToken, EditAssistantReq req) {
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_REMOVE_ASSISTANT, accessToken), req.toJson(), BaseResp.class);
	}

	/**
	 * 查询管理直播间小助手
	 *
	 * @param accessToken accessToken
	 * @param roomId      直播间ID
	 * @return {@link BaseResp}
	 */
	public static AssistantResp getAssistant(String accessToken, Integer roomId) {
		Map<String, Object> req = new HashMap<>();
		req.put("access_token", accessToken);
		req.put("roomId", roomId);
		return WxApiKit.getRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GET_ASSISTANT_LIST), req, AssistantResp.class);
	}

	/**
	 * 添加主播副号
	 *
	 * @param accessToken accessToken
	 * @param roomId      直播间ID
	 * @param username    用户微信号
	 * @return {@link BaseResp}
	 */
	public static BaseResp addSubAnchor(String accessToken, Integer roomId, String username) {
		JSONObject req = new JSONObject();
		req.put("roomId", roomId);
		req.put("username", username);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_ADD_SUBANCHOR), req.toJSONString(), BaseResp.class);
	}

	/**
	 * 修改主播副号
	 *
	 * @param accessToken accessToken
	 * @param roomId      直播间ID
	 * @param username    用户微信号
	 * @return {@link BaseResp}
	 */
	public static BaseResp modifySubAnchor(String accessToken, Integer roomId, String username) {
		JSONObject req = new JSONObject();
		req.put("roomId", roomId);
		req.put("username", username);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_MODIFY_SUBANCHOR), req.toJSONString(), BaseResp.class);
	}

	/**
	 * 删除主播副号
	 *
	 * @param accessToken accessToken
	 * @param roomId      直播间ID
	 * @return {@link BaseResp}
	 */
	public static BaseResp delSubAnchor(String accessToken, Integer roomId) {
		JSONObject req = new JSONObject();
		req.put("roomId", roomId);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_DEL_SUBANCHOR), req.toJSONString(), BaseResp.class);
	}

	/**
	 * 获取主播副号
	 *
	 * @param accessToken accessToken
	 * @param roomId      直播间ID
	 * @return {@link LiveSubAnchorResp}
	 */
	public static LiveSubAnchorResp getSubAnchor(String accessToken, Integer roomId) {
		Map<String, Object> req = new HashMap<>();
		req.put("access_token", accessToken);
		req.put("roomId", roomId);
		return WxApiKit.getRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GET_SHARE_CODE), req, LiveSubAnchorResp.class);
	}

	/**
	 * 开启/关闭直播间官方收录
	 *
	 * @param accessToken accessToken
	 * @param roomId      直播间ID
	 * @param flag        开启/关闭
	 * @return {@link BaseResp}
	 */
	public static BaseResp updateFeedPublic(String accessToken, Integer roomId, boolean flag) {
		JSONObject req = new JSONObject();
		req.put("roomId", roomId);
		req.put("isFeedsPublic", flag ? 1 : 0);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_UPDATE_FEED_PUBLIC, accessToken), req.toJSONString(), BaseResp.class);
	}

	/**
	 * 开启/关闭回放功能
	 *
	 * @param accessToken accessToken
	 * @param roomId      直播间ID
	 * @param flag        开启/关闭
	 * @return {@link BaseResp}
	 */
	public static BaseResp updateReplay(String accessToken, Integer roomId, boolean flag) {
		JSONObject req = new JSONObject();
		req.put("roomId", roomId);
		req.put("closeReplay", flag ? 0 : 1);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_UPDATE_REPLAY, accessToken), req.toJSONString(), BaseResp.class);
	}

	/**
	 * 开启/关闭客服功能
	 *
	 * @param accessToken accessToken
	 * @param roomId      直播间ID
	 * @param flag        开启/关闭
	 * @return {@link BaseResp}
	 */
	public static BaseResp updateKf(String accessToken, Integer roomId, boolean flag) {
		JSONObject req = new JSONObject();
		req.put("id", roomId);
		req.put("closeKf", flag ? 0 : 1);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_UPDATE_KF, accessToken), req.toJSONString(), BaseResp.class);
	}


	/**
	 * 开启/关闭直播间全局禁言
	 *
	 * @param accessToken accessToken
	 * @param roomId      直播间ID
	 * @param flag        开启/关闭
	 * @return {@link BaseResp}
	 */
	public static BaseResp updateComment(String accessToken, Integer roomId, boolean flag) {
		JSONObject req = new JSONObject();
		req.put("id", roomId);
		req.put("banComment", flag ? 0 : 1);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_UPDATE_COMMENT, accessToken), req.toJSONString(), BaseResp.class);
	}

	/**
	 * 设置小程序直播成员的管理员、运营者和主播角色
	 *
	 * @param accessToken accessToken
	 * @param userName    用户的微信号
	 * @param role        删除用户的角色; [1-管理员，2-主播，3-运营者]
	 * @return {@link BaseResp}
	 */
	public static BaseResp addRole(String accessToken, String userName, int role) {
		JSONObject req = new JSONObject();
		req.put("username", userName);
		req.put("role", role);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_ADD_ROLE, accessToken), req.toJSONString(), BaseResp.class);
	}


	/**
	 * 移除小程序直播成员的管理员、运营者和主播角色
	 *
	 * @param accessToken accessToken
	 * @param userName    用户的微信号
	 * @param role        删除用户的角色; [1-管理员，2-主播，3-运营者]
	 * @return {@link BaseResp}
	 */
	public static BaseResp delRole(String accessToken, String userName, int role) {
		JSONObject req = new JSONObject();
		req.put("username", userName);
		req.put("role", role);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_DEL_ROLE, accessToken), req.toJSONString(), BaseResp.class);
	}


	/**
	 * 查询小程序直播成员列表
	 *
	 * @param accessToken accessToken
	 * @param req         请求参数
	 * @return {@link LiveRoleResp}
	 */
	public static LiveRoleResp getRoleList(String accessToken, LiveRoleReq req) {
		Map<String, Object> map = req.toMap();
		map.put("access_token", accessToken);
		return WxApiKit.getRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GET_ROLE_LIST), map, LiveRoleResp.class);
	}


	/**
	 * 商品添加并提审
	 *
	 * @param accessToken AccessToken
	 * @param goods       商品信息
	 * @return {@link GoodsAddBaseResp}
	 */
	public static GoodsAddBaseResp add(String accessToken, Goods goods) {
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GOODS_ADD, accessToken), new GoodsInfoReq(goods).toJson(), GoodsAddBaseResp.class);
	}

	/**
	 * 更新商品信息<br>
	 * 审核通过的商品仅允许更新价格类型与价格，审核中的商品不允许更新，未审核的商品允许更新所有字段，只传入需要更新的字段
	 *
	 * @param accessToken AccessToken
	 * @param goodsInfo   商品信息
	 * @return {@link BaseResp}
	 */
	public static BaseResp update(String accessToken, GoodsInfoReq goodsInfo) {
		JSONObject req = new JSONObject();
		req.put("goodsInfo", goodsInfo);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GOODS_UPDATE, accessToken), req.toJSONString(), BaseResp.class);
	}


	/**
	 * 撤回直播商品的提审申请
	 *
	 * @param accessToken AccessToken
	 * @param req         请求参数
	 * @return {@link BaseResp}
	 */
	public static BaseResp resetAudit(String accessToken, GoodsEditReq req) {
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GOODS_RESET_AUDIT, accessToken), req.toJson(), BaseResp.class);
	}

	/**
	 * 对已撤回提审的商品再次发起提审申请
	 *
	 * @param accessToken AccessToken
	 * @param req         请求参数
	 * @return {@link AuditResp}
	 */
	public static AuditResp audit(String accessToken, GoodsEditReq req) {
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GOODS_AUDIT, accessToken), req.toJson(), AuditResp.class);
	}


	/**
	 * 删除【小程序直播】商品库中的商品，删除后直播间上架的该商品也将被同步删除，不可恢复
	 *
	 * @param accessToken AccessToken
	 * @param req         请求参数
	 * @return {@link BaseResp}
	 */
	public static BaseResp delete(String accessToken, GoodsEditReq req) {
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GOODS_DEL, accessToken), req.toJson(), BaseResp.class);
	}


	/**
	 * 获取商品的信息与审核状态
	 *
	 * @param accessToken AccessToken
	 * @param goodsIds    商品ID
	 * @return {@link GoodsInfoResp}
	 */
	public static GoodsInfoResp getGoodsInfo(String accessToken, int... goodsIds) {
		JSONObject req = new JSONObject();
		req.put("goods_ids", goodsIds);
		return WxApiKit.postRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GOODS_WARAHOUSE, accessToken), req.toJSONString(), GoodsInfoResp.class);
	}


	/**
	 * 获取商品列表
	 *
	 * @param accessToken AccessToken
	 * @param offset      分页条数起点
	 * @param limit       分页大小，默认30，不超过100
	 * @param status      商品状态，0：未审核。1：审核中，2：审核通过，3：审核驳回
	 * @return {@link GoodsInfoResp}
	 */
	public static GoodsInfoResp getGoodsList(String accessToken, Integer offset, Integer limit, Integer status) {
		Map<String, Object> req = new HashMap<>();
		req.put("offset", offset);
		req.put("limit", limit);
		req.put("status", status);
		req.put("access_token", accessToken);
		return WxApiKit.getRequest(WxApiKit.getUrl(WxApiEnum.LIVE_GOODS_APPROVED), req, GoodsInfoResp.class);
	}


}
