package net.javase.wechat.kit;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import net.javase.wechat.bean.wxa.WxaUserInfo;
import net.javase.wechat.enums.MediaType;
import net.javase.wechat.enums.WxApiEnum;
import net.javase.wechat.enums.WxDomain;
import net.javase.wechat.model.resp.BaseResp;
import net.javase.wechat.model.resp.MediaResp;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.io.File;
import java.nio.charset.StandardCharsets;
import java.security.AlgorithmParameters;
import java.security.Security;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * WxApiKit
 *
 * @author Frank
 */
@Slf4j
public class WxApiKit {

	/**
	 * 将map转换成url
	 *
	 * @param baseUrl {@link String} 请求URL
	 * @param map     {@link Map} map参数对象
	 * @return {@link String} 完整URL
	 */
	public static String getUrlByMap(String baseUrl, Map<String, Object> map) {
		if (map == null || map.size() < 1) {
			return baseUrl;
		}
		StringBuilder sb = new StringBuilder(baseUrl).append("?");
		for (Map.Entry<String, Object> entry : map.entrySet()) {
			sb.append(entry.getKey()).append("=").append(entry.getValue());
			sb.append("&");
		}
		String s = sb.toString();
		if (s.endsWith("&")) {
			s = s.substring(0, s.length() - 1);
		}
		return s;
	}

	/**
	 * 获取带token的请求地址
	 *
	 * @param wxApi       wxApi
	 * @param accessToken AccessToken
	 * @return {@link String}
	 */
	public static String getUrl(WxApiEnum wxApi, String accessToken) {
		return WxDomain.CHINA.getDomain() + wxApi.getApi() + "?access_token=" + accessToken;
	}

	/**
	 * 获取完整请求地址(不带token的)
	 *
	 * @param wxApi wxApi
	 * @return {@link String}
	 */
	public static String getUrl(WxApiEnum wxApi) {
		return WxDomain.CHINA.getDomain() + wxApi.getApi();
	}

	/**
	 * 公众号上传临时素材
	 *
	 * @param accessToken AccessToken
	 * @param type        媒体文件类型，分别有图片（image）、语音（voice）、视频（video）和缩略图（thumb）
	 * @param file        file
	 * @return {@link String}
	 */
	public static String uploadGetMediaId(String accessToken, String type, File file) {
		if (StrUtil.isBlank(accessToken) || StrUtil.isBlank(type)) {
			log.info("临时素材上传 ---- 参数错误");
			return null;
		}
		if (file == null || !file.exists()) {
			log.info("临时素材上传 ---- 找不到文件");
			return null;
		}
		String url = getUrl(WxApiEnum.MEDIA_UPLOAD, accessToken) + "&type=" + type;
		Map<String, Object> req = new HashMap<>();
		req.put("media", file);
		String res = HttpRequest.post(url).form(req).execute().body();
		log.info(" ----------- 临时素材上传结果 ------------- \n{}", res);
		MediaResp resp = JSONObject.parseObject(res, MediaResp.class);
		return resp.getMedia_id();
	}

	/**
	 * 公众号上传临时图片素材
	 *
	 * @param accessToken AccessToken
	 * @param file        图片文件
	 * @return {@link String}
	 */
	public static String uploadImgGetMediaId(String accessToken, File file) {
		return uploadGetMediaId(accessToken, MediaType.IMAGE.getValue(), file);
	}


	/**
	 * 生成JS_SDK Config需要的参数
	 *
	 * @param appId  appId
	 * @param url    url
	 * @param ticket js_ticket
	 * @return {@link Map}
	 */
	public static Map<String, String> createJsSdkConfig(String appId, String url, String ticket) {
		String nonce_str = IdUtil.fastSimpleUUID();
		String timestamp = Long.toString(System.currentTimeMillis() / 1000);
		Map<String, String> ret = new HashMap<>();
		String str = "jsapi_ticket=" + ticket + "&noncestr="
			+ nonce_str + "&timestamp=" + timestamp + "&url=" + url;

		String signature = SecureUtil.sha1(str);

		ret.put("url", url);
		ret.put("nonceStr", nonce_str);
		ret.put("timestamp", timestamp);
		ret.put("signature", signature);
		ret.put("appId", appId);
		return ret;
	}

	/**
	 * 微信小程序解密用户信息
	 *
	 * @param encryptedData encryptedData
	 * @param sessionKey    sessionKey
	 * @param iv            iv
	 * @return 解密的用户信息
	 */
	public static WxaUserInfo getUserInfo(String encryptedData, String sessionKey, String iv) {
		byte[] dataByte = Base64.decode(encryptedData);
		byte[] keyByte = Base64.decode(sessionKey);
		byte[] ivByte = Base64.decode(iv);
		try {
			int base = 16;
			if (keyByte.length % base != 0) {
				int groups = keyByte.length / base + 1;
				byte[] temp = new byte[groups * base];
				Arrays.fill(temp, (byte) 0);
				System.arraycopy(keyByte, 0, temp, 0, keyByte.length);
				keyByte = temp;
			}

			Security.addProvider(new BouncyCastleProvider());
			Cipher cipher = Cipher.getInstance("AES/CBC/PKCS7Padding", "BC");
			SecretKeySpec spec = new SecretKeySpec(keyByte, "AES");
			AlgorithmParameters parameters = AlgorithmParameters.getInstance("AES");
			parameters.init(new IvParameterSpec(ivByte));
			cipher.init(Cipher.DECRYPT_MODE, spec, parameters);// 初始化
			byte[] resultByte = cipher.doFinal(dataByte);
			if (Objects.nonNull(resultByte) && resultByte.length > 0) {
				String result = new String(resultByte, StandardCharsets.UTF_8);
				log.info("小程序解密用户信息： {}", result);
				return JSON.parseObject(result, WxaUserInfo.class);
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
		return null;
	}

	/**
	 * post请求
	 *
	 * @param url   请求地址
	 * @param data  请求数据
	 * @param clazz class
	 * @param <T>   T
	 * @return 响应结果
	 */
	public static <T extends BaseResp> T postRequest(String url, String data, Class<T> clazz) {
		String res = postRequest(url, data).body();
		log.info("\n--------------------- 微信公众号请求 ---------------------\nUrl: {}\nReq: {}\nResp: {}\n---------------------------------------------------",
			url.substring(0, url.indexOf("?")), data, res);
		return JSONObject.parseObject(res, clazz);
	}

	/**
	 * get请求
	 *
	 * @param url   请求地址
	 * @param data  Get请求参数
	 * @param clazz class
	 * @param <T>   T
	 * @return 响应结果
	 */
	public static <T extends BaseResp> T getRequest(String url, Map<String, Object> data, Class<T> clazz) {
		String res = getRequest(url, data).body();
		log.info("\n--------------------- 微信公众号请求 ---------------------\nUrl: {}\nReq: {}\nResp: {}\n---------------------------------------------------",
			url, data, res);
		return JSONObject.parseObject(res, clazz);
	}


	/**
	 * Post请求
	 *
	 * @param url  请求地址
	 * @param body 请求数据
	 * @return 响应结果
	 */
	public static HttpResponse postRequest(String url, String body) {
		return HttpRequest.post(url).body(body).execute();
	}

	/**
	 * Get请求
	 *
	 * @param url  请求地址
	 * @param data 请求参数
	 * @return 响应结果
	 */
	public static HttpResponse getRequest(String url, Map<String, Object> data) {
		return HttpRequest.get(url).form(data).execute();
	}
}

