package net.javase.wechat.pay;

import cn.hutool.core.io.FileUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.Method;
import cn.hutool.http.body.MultipartBody;
import cn.hutool.http.body.RequestBody;
import com.alibaba.fastjson.JSONObject;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.log4j.Log4j2;
import lombok.extern.slf4j.Slf4j;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.*;
import java.security.cert.*;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

/**
 * 微信V3接口
 *
 * @author Frank
 */
@Slf4j
@Getter
@Setter
public class WxHttpUtil {

	public static final String SCHEMA = "WECHATPAY2-SHA256-RSA2048 ";

	private String merchantId;

	private String certSerialNo;

	private PrivateKey privateKey;

	/**
	 * 平台证书序列号
	 */
	private String ptCertNo;

	public WxHttpUtil(String merNo, String certNo, PrivateKey privateKey) {
		this.merchantId = merNo;
		this.certSerialNo = certNo;
		this.privateKey = privateKey;
	}

	public WxHttpUtil(String merNo, String certNo, String keyFileName) throws IOException {
		this.merchantId = merNo;
		this.certSerialNo = certNo;
		this.privateKey = getPrivateKey(keyFileName);
	}

	/**
	 * 平台序列号
	 *
	 * @param certNo 平台证书号
	 */
	public void setPtCertNo(String certNo) {
		this.ptCertNo = certNo;
	}

	public String getAuthorization(String method, String url, String body) throws Exception {
		String nonceStr = WxPayKit.getNonceStr();
		long timestamp = System.currentTimeMillis() / 1000;
		String message = buildMessage(method, url, timestamp, nonceStr, body);
		String signature = sign(message.getBytes(StandardCharsets.UTF_8));

		return "mchid=\"" + merchantId + "\","
			+ "nonce_str=\"" + nonceStr + "\","
			+ "timestamp=\"" + timestamp + "\","
			+ "serial_no=\"" + certSerialNo + "\","
			+ "signature=\"" + signature + "\"";
	}

	public String sign(byte[] message) throws SignatureException, NoSuchAlgorithmException, InvalidKeyException {
		Signature sign = Signature.getInstance("SHA256withRSA");
		sign.initSign(privateKey);
		sign.update(message);
		return Base64.getEncoder().encodeToString(sign.sign());
	}


	/**
	 * 构造签名串
	 *
	 * @param method    Http请求方法
	 * @param url       请求地址
	 * @param timestamp 时间戳
	 * @param nonceStr  随机字符串
	 * @param body      请求内容
	 */
	public String buildMessage(String method, String url, long timestamp, String nonceStr, String body) {
//		HttpUrl httpurl = HttpUrl.parse(url);
//		assert httpurl != null;
//		String canonicalUrl = httpurl.encodedPath();
//		if (httpurl.encodedQuery() != null) {
//			canonicalUrl += "?" + httpurl.encodedQuery();
//		}

		return method + "\n"
			+ url + "\n"
			+ timestamp + "\n"
			+ nonceStr + "\n"
			+ body + "\n";
	}


	/**
	 * GET 请求
	 *
	 * @param url 请求地址
	 */
	public String get(String url) {
		try {
			String token = getAuthorization(Method.GET.name(), url, "");
			return HttpRequest.get(url)
				.header("Accept", "application/json")
				.header("Authorization", SCHEMA + token)
				.execute().body();
		} catch (Exception e) {
			log.error("WX_V3_GET请求出错", e);
			return null;
		}
	}

	public String get(String url, Map<String, String> params) {
		String resUrl = WxPayKit.getUrlStrByMap(url, params);
		return get(resUrl);
	}


	/**
	 * 微信V3 接口加密post请求
	 *
	 * @param url    请求地址
	 * @param params 请求参数
	 */
	public String post(String url, String params) {
		try {
			String token = getAuthorization(Method.POST.name(), url, params);
			return HttpRequest.post(url)
				.header("Accept", "application/json")
				.header("Authorization", SCHEMA + token)
				.header("Wechatpay-Serial", ptCertNo)
				.body(params)
				.execute().body();
		} catch (Exception e) {
			log.error("WX_V3_POST请求出错", e);
			return null;
		}
	}

	public String post(String url, JSONObject params) {
		return post(url, params.toJSONString());
	}


	/**
	 * 获取平台证书
	 *
	 * @param apiKey3 API_V3密钥
	 */
//	public X509Certificate getPlatformCert(String apiKey3) throws GeneralSecurityException {
//		JSONObject res = JSONObject.parseObject(get(WxV3Constant.GET_CERT_URL));
//		JSONObject result = res.getJSONArray("data").getJSONObject(0).getJSONObject("encrypt_certificate");
//		AesUtil aesUtil = new AesUtil(apiKey3.getBytes(StandardCharsets.UTF_8));
//		String publicKey = aesUtil.decryptToString(
//			result.getString("associated_data").getBytes(StandardCharsets.UTF_8),
//			result.getString("nonce").getBytes(StandardCharsets.UTF_8),
//			result.getString("ciphertext")
//		);
//		X509Certificate certificate = getCertificate(new ByteArrayInputStream(publicKey.getBytes()));
//		this.ptCertNo = certificate.getSerialNumber().toString(16).toUpperCase();
//		log.info("平台证书序列号: {}", this.ptCertNo);
//		return certificate;
//	}


//	/**
//	 * 图片上传
//	 *
//	 * @param filePath 图片文件地址
//	 */
//	public String uploadImage(String filePath) {
//		try {
//			File file = new File(filePath);
//			String filename = file.getName();
//			String fileSha256 = DigestUtils.sha256Hex(new FileInputStream(file));
//			OkHttpClient client = new OkHttpClient();
//			String params = "{\"filename\":\"" + filename + "\",\"sha256\":\"" + fileSha256 + "\"}";
//			String token = getAuthorization("POST", WxV3Constant.UPLOAD_URL, params);
//			RequestBody fileBody = RequestBody.create(MediaType.parse("image/jpg"), file);
//			MultipartBody multipartBody = new MultipartBody.Builder()
//				.setType(MultipartBody.FORM)
//				.addFormDataPart("file", filename, fileBody)
//				.addFormDataPart("meta", params)
//				.build();
//
//			Request request = new Request.Builder()
//				.url(WxV3Constant.UPLOAD_URL)
//				.addHeader("Accept", "application/json")
//				.addHeader("Authorization", SCHEMA + token)
//				.addHeader("Content-Type", "multipart/form-data;boundary=\"boundary\"")
//				.post(multipartBody)
//				.build();
//			Call call = client.newCall(request);
//			Response response = call.execute();
//			return response.body() != null ? response.body().string() : null;
//		} catch (Exception e) {
//			log.error("WX_V3_图片上传出错", e);
//			return null;
//		}
//	}
//
//	/**
//	 * 上传图片, 返回media_id
//	 *
//	 * @param filePath 文件路径
//	 * @return media_id
//	 */
//	public String upload(String filePath) {
//		try {
//			File file = new File(filePath);
//			String filename = file.getName();
//			String fileSha256 = DigestUtils.sha256Hex(new FileInputStream(file));
//			String params = "{\"filename\":\"" + filename + "\",\"sha256\":\"" + fileSha256 + "\"}";
//			String token = getAuthorization(Method.POST.name(), WxV3Constant.UPLOAD_URL, params);
//			HashMap<String, Object> paramMap = new HashMap<>();
//			paramMap.put("file", FileUtil.file(filePath));
//			paramMap.put("meta", params);
//			String result = HttpRequest.post(WxV3Constant.UPLOAD_URL)
//				.header("Accept", "application/json")
//				.header("Authorization", SCHEMA + token)
//				.header("Content-Type", "multipart/form-data;boundary=\"boundary\"")
//				.form(paramMap)
//				.execute().body();
//			log.info("微信V3图片上传: {}", result);
//			return JSONObject.parseObject(result).getString("media_id");
//		} catch (Exception e) {
//			log.error("WX_V3_图片上传出错", e);
//			return null;
//		}
//	}


	/**
	 * 微信V3接口铭感信息加密
	 *
	 * @param message     加密消息
	 * @param certificate 证书
	 */
	public static String rsaEncryptOAEP(String message, X509Certificate certificate)
		throws IllegalBlockSizeException {
		try {
			Cipher cipher = Cipher.getInstance("RSA/ECB/OAEPWithSHA-1AndMGF1Padding");
			cipher.init(Cipher.ENCRYPT_MODE, certificate.getPublicKey());

			byte[] data = message.getBytes(StandardCharsets.UTF_8);
			byte[] cipherData = cipher.doFinal(data);
			return Base64.getEncoder().encodeToString(cipherData);
		} catch (NoSuchAlgorithmException | NoSuchPaddingException e) {
			throw new RuntimeException("当前Java环境不支持RSA v1.5/OAEP", e);
		} catch (InvalidKeyException e) {
			throw new IllegalArgumentException("无效的证书", e);
		} catch (IllegalBlockSizeException | BadPaddingException e) {
			throw new IllegalBlockSizeException("加密原串的长度不能超过214字节");
		}
	}


	/**
	 * 获取私钥。
	 *
	 * @param filename 私钥文件路径  (required)  apiclient_key.pem
	 * @return 私钥对象
	 */
	public static PrivateKey getPrivateKey(String filename) throws IOException {
		String content = new String(Files.readAllBytes(Paths.get(filename)), StandardCharsets.UTF_8);
		try {
			String privateKey = content.replace("-----BEGIN PRIVATE KEY-----", "")
				.replace("-----END PRIVATE KEY-----", "")
				.replaceAll("\\s+", "");

			KeyFactory kf = KeyFactory.getInstance("RSA");
			return kf.generatePrivate(
				new PKCS8EncodedKeySpec(Base64.getDecoder().decode(privateKey)));
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException("当前Java环境不支持RSA", e);
		} catch (InvalidKeySpecException e) {
			throw new RuntimeException("无效的密钥格式");
		}
	}

	/**
	 * 获取证书。
	 *
	 * @param filename 证书文件路径  (required)  apiclient_cert.pem
	 * @return X509证书
	 */
	public static X509Certificate getCertificate(String filename) throws IOException {
		return getCertificate(new FileInputStream(filename));
	}

	public static X509Certificate getCertificate(InputStream fis) {
		try {
			CertificateFactory cf = CertificateFactory.getInstance("X509");
			X509Certificate cert = (X509Certificate) cf.generateCertificate(fis);
			cert.checkValidity();
			return cert;
		} catch (CertificateExpiredException e) {
			throw new RuntimeException("证书已过期", e);
		} catch (CertificateNotYetValidException e) {
			throw new RuntimeException("证书尚未生效", e);
		} catch (CertificateException e) {
			throw new RuntimeException("无效的证书文件", e);
		}
	}


//	/**
//	 * 获取微信MediaID
//	 *
//	 * @param fileKey oss 图片文件key
//	 * @return {@link String} MediaID
//	 */
//	public String getWxMediaID(String fileKey) throws PayException {
//		if (StrKit.notBlank(fileKey)) {
//			String filename = PathKit.getWebRootPath() + "/upload/temp/"
//				+ fileKey.substring(fileKey.lastIndexOf("/") + 1);
//			//AliOssKit.init().get(fileKey, filename);
//			TxCosKit.get(fileKey, filename);
//			File file = new File(filename);
//			if (file.length() >= 2 * 1024 * 1024) {
//				throw new PayException("图片大小超过2M, 请检查图片");
//			}
//			if (!file.exists()) {
//				throw new PayException("图片文件不存在, 请检查图片");
//			}
//			return upload(filename);
//		}
//		return "";
//	}
//
//
//	/**
//	 * 获取商户结算信息
//	 *
//	 * @return {@link SettlementInfo} 结算规则信息
//	 */
//	public SettlementInfo getSettleInfo(Record record) throws PayException {
//		//个体户
//		SettlementInfo info = SettlementInfo.builder().settlement_id("719")
//			.qualification_type("零售批发/生活娱乐/其他")
//			.activities_id("20191030111cff5b5e")
//			.activities_rate("0.38")
//			.build();
//		//企业
//		if ("2".equals(record.getStr("one_type"))) {
//			info.setSettlement_id("716");
//			info.setQualification_type("零售批发/生活娱乐/网上商城/其他");
//		}
//
//		//餐饮
//		if (StrKit.notBlank(record.getStr("business_sector")) && record.getStr("business_sector").contains("餐饮")) {
//			String qualify = getWxMediaID(record.getStr("special_qualification"));
//			info.setQualification_type("餐饮");
//			info.setQualifications(new String[]{qualify});
//		}
//		return info;
//	}

}
