package net.javase.wechat.pay;

import cn.hutool.core.codec.Base64Decoder;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.core.util.URLUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.GCMParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.io.*;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.*;
import java.security.cert.*;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

/**
 * WxPayKit
 *
 * @author Frank
 */
@Slf4j
public class WxPayKit {

	public static final String DEFAULT_AUTH = "WECHATPAY2-SHA256-RSA2048";

	public static String getAuthorization(V3PayRequest request) throws Exception {
		String nonceStr = getNonceStr();
		long timestamp = System.currentTimeMillis() / 1000;
		String url = request.getApi();
		if (request.getQueryParams() != null && request.getQueryParams().size() > 0) {
			url = getUrlStrByMap(url, request.getQueryParams());
		}
		String message = buildMessage(request.getMethod().toString(), url, timestamp, nonceStr, request.getBody());
		String signature = sign(message.getBytes(StandardCharsets.UTF_8), request.getPrivateKey());

		return request.getAuthType() + " mchid=\""
			+ request.getMchId() + "\","
			+ "nonce_str=\"" + nonceStr + "\","
			+ "timestamp=\"" + timestamp + "\","
			+ "serial_no=\"" + request.getCertSerialNo() + "\","
			+ "signature=\"" + signature + "\"";
	}

	/**
	 * 构造签名串
	 *
	 * @param method    Http请求方法
	 * @param url       请求地址
	 * @param timestamp 时间戳
	 * @param nonceStr  随机字符串
	 * @param body      请求内容
	 */
	public static String buildMessage(String method, String url, long timestamp, String nonceStr, String body) {
		return method + "\n"
			+ url + "\n"
			+ timestamp + "\n"
			+ nonceStr + "\n"
			+ body + "\n";
	}

	public static String sign(byte[] message, PrivateKey privateKey) throws SignatureException, NoSuchAlgorithmException, InvalidKeyException {
		Signature sign = Signature.getInstance("SHA256withRSA");
		sign.initSign(privateKey);
		sign.update(message);
		return Base64.getEncoder().encodeToString(sign.sign());
	}

	/**
	 * 获取平台证书
	 *
	 * @param apiKey3 API_V3密钥
	 */
	public X509Certificate getPlatformCert(String apiKey3, byte[] associatedData, byte[] nonce, String cipherText) throws GeneralSecurityException {
		Cipher cipher = Cipher.getInstance("AES/GCM/NoPadding");
		SecretKeySpec key = new SecretKeySpec(apiKey3.getBytes(StandardCharsets.UTF_8), "AES");
		GCMParameterSpec spec = new GCMParameterSpec(128, nonce);
		cipher.init(Cipher.DECRYPT_MODE, key, spec);
		cipher.updateAAD(associatedData);
		String publicKey = new String(cipher.doFinal(Base64Decoder.decode(cipherText)), StandardCharsets.UTF_8);

		X509Certificate certificate = getCertificate(new ByteArrayInputStream(publicKey.getBytes()));
		String platSerialNo = certificate.getSerialNumber().toString(16).toUpperCase();
		log.info("平台证书序列号: {}", platSerialNo);
		return certificate;
	}

	public static X509Certificate getCertificate(InputStream fis) {
		try {
			CertificateFactory cf = CertificateFactory.getInstance("X509");
			X509Certificate cert = (X509Certificate) cf.generateCertificate(fis);
			cert.checkValidity();
			return cert;
		} catch (CertificateExpiredException e) {
			throw new RuntimeException("证书已过期", e);
		} catch (CertificateNotYetValidException e) {
			throw new RuntimeException("证书尚未生效", e);
		} catch (CertificateException e) {
			throw new RuntimeException("无效的证书文件", e);
		}
	}

	/**
	 * 微信V3接口铭感信息加密
	 *
	 * @param message     加密消息
	 * @param certificate 证书
	 */
	public static String rsaEncryptOAEP(String message, X509Certificate certificate)
		throws IllegalBlockSizeException {
		try {
			Cipher cipher = Cipher.getInstance("RSA/ECB/OAEPWithSHA-1AndMGF1Padding");
			cipher.init(Cipher.ENCRYPT_MODE, certificate.getPublicKey());

			byte[] data = message.getBytes(StandardCharsets.UTF_8);
			byte[] cipherData = cipher.doFinal(data);
			return Base64.getEncoder().encodeToString(cipherData);
		} catch (NoSuchAlgorithmException | NoSuchPaddingException e) {
			throw new RuntimeException("当前Java环境不支持RSA v1.5/OAEP", e);
		} catch (InvalidKeyException e) {
			throw new IllegalArgumentException("无效的证书", e);
		} catch (IllegalBlockSizeException | BadPaddingException e) {
			throw new IllegalBlockSizeException("加密原串的长度不能超过214字节");
		}
	}

	/**
	 * 获取私钥。
	 *
	 * @param filename 私钥文件路径  (required)  apiclient_key.pem
	 * @return 私钥对象
	 */
	public static PrivateKey getPrivateKey(String filename) throws IOException {
		String content = new String(Files.readAllBytes(Paths.get(filename)), StandardCharsets.UTF_8);
		return getPrivateKeyByContent(content);
	}

	public static PrivateKey getPrivateKey(File file) throws IOException {
		String content = new String(FileUtil.readBytes(file), StandardCharsets.UTF_8);
		return getPrivateKeyByContent(content);
	}

	public static PrivateKey getPrivateKeyByContent(String content) {
		if (StrUtil.isBlank(content)) {
			throw new NullPointerException("content is null");
		}
		try {
			String privateKey = content.replace("-----BEGIN PRIVATE KEY-----", "")
				.replace("-----END PRIVATE KEY-----", "")
				.replaceAll("\\s+", "");

			KeyFactory kf = KeyFactory.getInstance("RSA");
			return kf.generatePrivate(
				new PKCS8EncodedKeySpec(Base64Decoder.decode(privateKey)));
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException("当前Java环境不支持RSA", e);
		} catch (InvalidKeySpecException e) {
			throw new RuntimeException("无效的密钥格式");
		}
	}

	/**
	 * 获取证书。
	 *
	 * @param filename 证书文件路径  (required)  apiclient_cert.pem
	 * @return X509证书
	 */
	public static X509Certificate getCertificate(String filename) throws IOException {
		return getCertificate(new FileInputStream(filename));
	}

	public static String getNonceStr() {
		return IdUtil.fastSimpleUUID().toUpperCase();
	}

	/**
	 * 将map转换成url
	 *
	 * @param baseUrl {@link String} 请求URL
	 * @param map     {@link Map} map参数对象
	 * @return {@link String} 完整URL
	 */
	public static String getUrlStrByMap(String baseUrl, Map<String, String> map) {
		if (map == null) {
			return "";
		}
		StringBuilder sb = new StringBuilder();
		for (Map.Entry<String, String> entry : map.entrySet()) {
			sb.append(entry.getKey()).append("=")
				.append(URLUtil.encode(entry.getValue())).append("&");
		}
		String s = sb.toString();
		return baseUrl + "?" + s.substring(0, s.length() - 1);
	}

	public static JSONObject request(V3PayRequest request) {
		switch (request.getMethod()) {
			case GET:
				return JSONObject.parseObject(get(request));
			case POST:
				return JSONObject.parseObject(post(request));
			case UPLOAD:
				return JSONObject.parseObject(upload(request));
			default:
				throw new IllegalStateException("Unexpected value: " + request.getMethod().toString());
		}
	}

	/**
	 * GET 请求
	 *
	 * @param request 请求信息
	 * @return 响应body
	 */
	private static String get(V3PayRequest request) {
		try {
			return HttpRequest.get(request.getDomain() + request.getApi())
				.charset(StandardCharsets.UTF_8)
				.header("Accept", "application/json")
				.header("Authorization", getAuthorization(request))
				.execute().body();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * 微信V3 接口加密post请求
	 *
	 * @param request 请求信息
	 * @return 响应body
	 */
	private static String post(V3PayRequest request) {
		try {
			return HttpRequest.post(request.getDomain() + request.getApi())
				.charset(StandardCharsets.UTF_8)
				.header("Accept", "application/json")
				.header("Authorization", getAuthorization(request))
				.header("Wechatpay-Serial", request.getPlatSerialNo())
				.body(request.getBody())
				.execute().body();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * 上传图片, 返回media_id
	 *
	 * @return media_id
	 */
	private static String upload(V3PayRequest request) {
		try {
			JSONObject params = new JSONObject();
			params.put("filename", request.getUploadFile().getName());
			params.put("sha256", SecureUtil.sha256(request.getUploadFile()));
			HashMap<String, Object> paramMap = new HashMap<>();
			paramMap.put("file", request.getUploadFile());
			paramMap.put("meta", params);
			request.setBody(params.toJSONString());
			return HttpRequest.post(request.getDomain() + request.getApi())
				.charset(StandardCharsets.UTF_8)
				.header("Accept", "application/json")
				.header("Authorization", getAuthorization(request))
				.header("Content-Type", "multipart/form-data;boundary=\"boundary\"")
				.form(paramMap)
				.execute().body();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

}
